#include <memory> // auto_ptr
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <stdlib.h> // getenv()
#include "url_ssh.hpp"
#include "pstreams.hpp"
#include <s11n.net/s11n/s11n_debuggering_macros.hpp>
#include <s11n.net/s11n/plugin/path_finder.hpp>
#include <s11n.net/s11n/io/url.hpp>

#include <s11n.net/s11n/classload.hpp>
#include <s11n.net/s11n/export.hpp> // unfortunate kludge


namespace s11n { namespace io { namespace pstreams {

	using ::s11n::io::url_parser;

	ssh_stream_factory::ssh_stream_factory() : ::s11n::io::url_stream_factory()
	{
	}

	ssh_stream_factory::~ssh_stream_factory()
	{
	}

	std::ostream *
	ssh_stream_factory::do_get_ostream( url_parser const & url ) const
	{
		if( ! url.good() ) return 0;
		std::string cmd = "ssh -C ";

		if( 0 != url.port() )
		{
			std::ostringstream os;
			os << std::dec << url.port();
			cmd += "-p "+os.str();
		}

		std::string foo( url.user() );
		if( ! foo.empty() )
		{
			cmd += foo + "@";
		}
		cmd += url.host() + " ";
		char quote = '\'';
		foo = url.path();
		if( foo.empty() )
		{
			return 0;
		}
		if( std::string::npos != foo.find('\'') )
		{
			quote = '"';
		}
		cmd += quote;
		cmd += "cat > "+foo;
		cmd += quote;
		//cmd += " 2>/dev/null";
		std::ostream * is = new redi::opstream( cmd );
		if( ! is->good() )
		{
			delete is;
			is = 0;
		}
		return is;
	}

	std::istream *
	ssh_stream_factory::do_get_istream( url_parser const & url ) const
	{
		if( ! url.good() ) return 0;
		std::string cmd = "ssh -C ";
		std::string foo( url.user() );
		if( ! foo.empty() )
		{
			cmd += foo + "@";
		}
		cmd += url.host() + " ";
		char quote = '\'';
		foo = url.path();
		if( foo.empty() )
		{
			return 0;
		}
		if( std::string::npos != foo.find('\'') )
		{
			quote = '"';
		}
		cmd += quote;
		cmd += "cat "+foo;
		cmd += quote;
		//cmd += " 2>/dev/null";
		std::istream * is = new redi::ipstream( cmd );
		if( ! is->good() )
		{
			delete is;
			is = 0;
		}
		return is;
	}


	void pstreams_ssh_static_init()
	{
		/******
		       Register with the classloader and set up some classloader aliases...
		       only if we find the ssh app.
		******/
		::s11n::plugin::path_finder p;
		char * path = ::getenv("PATH");
		p.path( path ? path : "/bin:/usr/bin:/usr/local/bin" );
#if defined(WIN32)
		p.extensions( ".exe" );
#endif
		std::string file = p.find( "ssh" );
		if( file.empty() )
		{
			CERR << "ssh not found in $PATH. ssh support disabled.\n";
			return;
		}
		s11n::io::url_stream_factory::register_factory_for_scheme<ssh_stream_factory>("ssh");
		s11n::io::url_stream_factory::register_factory_for_scheme<ssh_stream_factory>("scp");
		//typedef s11n::fac::factory_mgr<s11n::io::url_stream_factory> FacT;
		//s11n::fac::register_subtype<s11n::io::url_stream_factory, ssh_stream_factory>( "ssh" );
		//FacT::instance().alias( "scp", "ssh" );
	}

	bool pstreams_ssh_static_init_placeholder = (pstreams_ssh_static_init(),true);

}}} // namespaces
